==> bundle-0.0.7.tgz: package/lib/bundle.js <==

/*!
 * bundle
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

/**
 * Library version.
 */
exports.version = '0.0.6';

/**
 * Module dependencies.
 */
var jade = require('jade')
  , fs = require('fs')
  , exec = require('child_process').exec
  , Log = require('log')


 /**
 * Exports.
 */
exports.Bundle = Bundle;

exports.createBundle = function (opts) {
  return new Bundle(opts);
};


/**
 * Bundle constructor
 *
 * options parameters:
 * - views: templates files default folder
 * - tmp: temporal proccessing folder
 * - gzip: gzip command
 * - 'view engine': view engine
 * - extension: extension file
 * - encoding
 * - tmpPackname: temporal package name
 * - addLogFile: defines add or not a log file
 *
 * @api private
 */
function Bundle (opts) {
  // default options
  this.options = {
      views: 'views'
    , tmp: '/tmp'
    , gzip: 'zip'
    , 'view engine': 'jade'
    , extension: 'html'
    , encoding: 'utf8'
    , tmpPackname: 'bundle'
    , addLogFile: true
    , depth: 0
  };

  // mixing options
  for(var k in opts)
    this.options[k] = opts[k];

  // define layouts and partials folders
  this.options.layouts = this.options.layouts || this.options.views;
  this.options.partials = this.options.partials || this.options.layouts;
  
  // initial configs
  this.generated = false;
  this.filename = null;
  this.files = {};
  this.filesCount = 0;

  this.hash = 'node-bundle-' + String(Math.random()).substr(2);
};


/**
 * Define a file
 *
 * @api private
 */
Bundle.prototype.file = function () {
  var args = arguments
    , name = args[0]
    , fn = typeof args[1] == 'string' ?
        // fake function used in htmlString 
        function(a, b){ b(' static: no-template', {}); } : 
        (typeof args[1] == 'function' ? args[1] : args[2])

    , options = typeof args[1] == 'object' ? args[1] : {}
    , text = typeof args[1] == 'string' ? args[1] : false
    
  this.files[name] = {name: name, fn:fn, options: options, text: text};
  this.filesCount++;
};

Bundle.prototype.compress = function (callback) {
  var self = this;

  // make compressed package
  this.filename = this.bundlePath + '/' + this.options.tmpPackname + '.zip';
  var zipCmd = '(cd ' + exports.strEscape(this.bundlePath) + ' ; '
              + 'zip -r ' + exports.strEscape(this.options.tmpPackname + '.zip') + ' ./ )'

  self.log('compresing: ' + zipCmd);

  exec(zipCmd, function (err, stdout, stderr) {
    if (err) return callback(err);
    callback(null, self.filename);
  });
}

/**
 * Generates the structure in a tmp location and gzips it
 *
 * @api private
 */
Bundle.prototype.generate = function (params, callback) {
  var self = this;

  // options alias
  var _ops = this.options;

  var createTreePackage = function(fn) {
    // create tmpbundle folder
    self.bundlePath = _ops.tmp + '/' + self.hash;
    self.createFolder(self.bundlePath, function() {
      // log
      self.log('create tmp folder: ' + self.bundlePath);

      fn();
    });

  }


  var renderLayout = function(layout, opts, html, callback) {
    if(!layout) return callback(null);
    delete opts.layout;
    
    var layoutFilename =_ops.layouts + layout
      , options = {
          body: html
        }

    // mixing options
    for(var k in opts) options[k] = opts[k];
    
    jade.renderFile(layoutFilename, { locals: options }, function(err, html) {
      if(err) return callback(err);

      self.log('rendering layout: ' + layoutFilename);
      return callback(null, html);
    });
  }


  // if this.generated = true calls immediately with this.filename
  // gzips it
  // calls the function with the file location
  if(self.generated){
    callback(null, self.filename);
  }
  else {
    createTreePackage(function() {
      var fileCounter = self.filesCount;

      // outputs everything to a temporary directory (by calling all the file
      // routes in paralell and keeping a counter of completion)
      for (var filename in self.files) {

        (function (file) {
          var saveFilename = self.bundlePath + '/' + file.name + '.' + _ops.extension
            , templateCounter = 1
            , withPartials = false
            , renderedFile = '';

          // execute file function
          file.fn(params, function(template, options) {

            // partial elements
            var rendersStack = {}
              , renNumber = 0
              , seed = 'partial-' + (0|Math.random()*1e10)
              , getId = function(n) { return seed + '-' + n; }
 
            // partial rendering function
            options.partial = function(){
              withPartials = true;
              templateCounter++;

              var args = arguments
                , name = args[0]
                , callback = typeof args[1] == 'function' ? args[1] : args[2]
                , locals = typeof args[1] == 'object' ? args[1] : {}

              // mixing options
              for(var k in locals) options[k] = locals[k];

              var renderId = getId(renNumber);
              renNumber++;

              var partialFilename = _ops.partials + name + '.' + _ops['view engine'];

              jade.renderFile(partialFilename, { locals: options }, function(err, html) {
                if(err) return saveFile(err);

                rendersStack[renderId] = {html: html}
                --templateCounter || saveFile();
              });

              return '{' + renderId + '}';
            }

            var saveFile = function (err) {
              if(err) return callback(err);

              // replace partials
              if(withPartials)
                for(var id in rendersStack)
                  renderedFile = renderedFile.replace('{' + id + '}', rendersStack[id].html);

              fs.writeFile(saveFilename, renderedFile, _ops.encoding, function(err) {
                if (err) return callback(err);

                self.log('save file: ' + saveFilename);
                --fileCounter || (function () {
                  // sets this.generated = true
                  self.generated = true;
                  self.compress(callback);
                })();
              });
            }

            var renderFile = function (name, options, file, callback) {
              if(file.text) return callback(null, file.text);
              jade.renderFile(name, options, callback);
            }


            var viewsFilename =_ops.views + template;
            // render html file
            renderFile(viewsFilename, { locals: options }, file, function(err, html) {
              if (err) return callback(err);

              self.log('render file: ' + viewsFilename);

              renderLayout(file.options.layout, options, html, function(err, htmlLayout) {
                if (err) return callback(err);

                renderedFile = html;

                // replace html by htmlLayout
                if(htmlLayout) {
                  renderedFile = htmlLayout;
                  self.log('decoration: insert html into layout');
                }
              
                // create folders for file ?
                var rePath = file.name.match(/(.+)\/(\w*$)/)
                  , addFolder = rePath ? self.bundlePath + '/' + rePath[1] : false;

                self.createFolder(addFolder, function(folder) {

                  // save html in file (without partials)
                  --templateCounter || saveFile();
                });
              });
            });
          });

        })(self.files[filename]);

      };

    });

  }
};


/**
 * Generates a bundle and saves it to a custom file
 *
 * @api private
 */
Bundle.prototype.save = function (params, customFile, callback) {
  var self = this;
  this.generate(params, function(err, file){
    if(err) return callback(err);

    var withPath = customFile.match(/[\w|\/].*\//)
      , path = withPath.length ? withPath[0].substr(0, withPath[0].length - 1) : null;

    self.createFolder(path, function(err) {
      if(err) return callback(err);
      var mvFileCmd = 'mv ' + exports.strEscape(self.filename) + ' ' + customFile;

      exec(mvFileCmd, function (err, stdout, stderr) {
        if(err) return callback(err);

        self.log('moving file: ' + mvFileCmd);

        callback(null, customFile);
      });

    });

  });
};


/**
 * Generates a bundle and gets the buffer
 *
 * @param private
 */
Bundle.prototype.get = function (params, callback) {
  var self = this;
  this.generate(params, function() {

    // reads the file ...
    fs.readFile(self.filename, function(err, data) {
      if(err) return callback(err);

      // sets this.generated = false
      self.generated = false;

      // removes the file
      var rmTmpCmd = 'rm ' + exports.strEscape(self.bundlePath) + ' -R';
      exec(rmTmpCmd, function (err, stdout, stderr) {
        if(err) return callback(err);
        
        // ... and passes the buffer to fn
        callback(null, data);
      });
      
    });

  });

};


/**
 * log() util method
 **/
Bundle.prototype.log = function (msg) {
  if(!this.options.addLogFile) return null;
  if(!this.objLog)
    this.objLog = new Log(Log.INFO, fs.createWriteStream(this.bundlePath + '/' + this.options.tmpPackname + '.log'));
  
  this.objLog.info(msg);
}


/**
 * createFolder() util method
 **/
Bundle.prototype.createFolder = function(path, fn) {
  if(!path) return fn('');

  var mkdirCmd = 'mkdir ' + exports.strEscape(path) + ' -p';

  exec(mkdirCmd, function (err, stdout, stderr) {
    if(err) return fn(err);
    fn(null, path);
  });

}


Bundle.prototype.cleanTmpFolder = function (callback) {

  var rmTmpCmd = 'rm ' + exports.strEscape(this.bundlePath) + ' -R';
  exec(rmTmpCmd, function(err, stdout, stderr) {
    if(err) return callback(err);
    if(stderr) return callback(err, stderr);

    if(callback) callback();
  })

}


/**
 * Escape the given shell `arg`.
 *
 * @param {String} arg
 * @return {String}
 * @api public
 */
exports.strEscape = function (arg) {
  return '"' + String(arg).replace(/"/g, '\\"') + '"';
};

==> bundle-0.0.8.tgz: package/lib/bundle.js <==

/*!
 * bundle
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

/**
 * Library version.
 */
 
exports.version = '0.0.8';

/**
 * Module dependencies.
 */
 
var jade = require('jade')
  , fs = require('fs')
  , exec = require('child_process').exec
  , log = require('log')

 /**
 * Exports.
 */
 
exports.Bundle = Bundle;

exports.createBundle = function (opts) {
  return new Bundle(opts);
};

/**
 * Bundle constructor
 *
 * options parameters:
 * - views: templates files default folder
 * - tmp: temporal proccessing folder
 * - gzip: gzip command
 * - 'view engine': view engine
 * - extension: extension file
 * - encoding
 * - tmpPackname: temporal package name
 * - addLogFile: defines add or not a log file
 *
 * @api private
 */
 
function Bundle (opts) {
  // default options
  this.options = {
      views: 'views'
    , tmp: '/tmp'
    , gzip: 'zip'
    , 'view engine': 'jade'
    , extension: 'html'
    , encoding: 'utf8'
    , tmpPackname: 'bundle'
    , addLogFile: true
    , depth: 0
  };

  // mixing options
  for(var k in opts)
    this.options[k] = opts[k];

  // define layouts and partials folders
  this.options.layouts = this.options.layouts || this.options.views;
  this.options.partials = this.options.partials || this.options.layouts;
  
  // initial configs
  this.generated = false;
  this.filename = null;
  this.files = {};
  this.filesCount = 0;

  this.hash = 'node-bundle-' + String(Math.random()).substr(2);
};

/**
 * Defines a file handler.
 *
 * @api private
 */
 
Bundle.prototype.file = function () {
  var args = arguments
    , name = args[0]
    , fn = typeof args[1] == 'string' ?
        // fake function used in htmlString 
        function(a, b){ b(' static: no-template', {}); } : 
        (typeof args[1] == 'function' ? args[1] : args[2])

    , options = typeof args[1] == 'object' ? args[1] : {}
    , text = typeof args[1] == 'string' ? args[1] : false
    
  this.files[name] = {name: name, fn:fn, options: options, text: text};
  this.filesCount++;
};

/**
 * Compresses the file.
 *
 * @api private
 */

Bundle.prototype.compress = function (callback) {
  var self = this;

  // make compressed package
  this.filename = this.bundlePath + '/' + this.options.tmpPackname + '.zip';
  var zipCmd = '(cd ' + escape(this.bundlePath) + ' ; '
              + 'zip -r ' + escape(this.options.tmpPackname + '.zip') + ' ./ )'

  self.log('compressing:', zipCmd);

  exec(zipCmd, function (err, stdout, stderr) {
    if (err) return callback(err);
    callback(null, self.filename);
  });
}

/**
 * Generates the structure in a tmp location and gzips it
 *
 * @api private
 */
 
Bundle.prototype.generate = function (params, callback) {
  var self = this;

  // options alias
  var _ops = this.options;

  // creates the directory tree structure
  var createTreePackage = function(fn) {
    // create tmpbundle folder
    self.bundlePath = _ops.tmp + '/' + self.hash;
    self.createFolder(self.bundlePath, function(err, folder) {
      if (err) return callback(err);

      self.log('create tmp folder:', self.bundlePath);

      fn();
    });
  }

  // render layout decoration
  var renderLayout = function(layout, opts, html, callback) {
    if (!layout) return callback(null);
    delete opts.layout;
    
    var layoutFilename =_ops.layouts + layout
      , options = {
          body: html
        }

    // mixing options
    for(var k in opts)
      options[k] = opts[k];
    
    jade.renderFile(layoutFilename, { locals: options }, function(err, html) {
      if (err) return callback(err);

      self.log('rendering layout:', layoutFilename);
      return callback(null, html);
    });
  }

  // if this.generated = true calls immediately with this.filename
  // gzips it
  // calls the function with the file location
  if (self.generated){
    callback(null, self.filename);
  } else {
    createTreePackage(function() {
      var fileCounter = self.filesCount;

      // outputs everything to a temporary directory (by calling all the file
      // routes in paralell and keeping a counter of completion)
      for (var filename in self.files) {

        (function (file) {
          var saveFilename = self.bundlePath + '/' + file.name + '.' + _ops.extension
            , templateCounter = 1
            , withPartials = false
            , renderedFile = '';

          // execute file function
          file.fn(params, function(template, options) {

            // partial elements
            var rendersStack = {}
              , renNumber = 0
              , seed = 'partial-' + (0|Math.random()*1e10)
              , getId = function(n) { return seed + '-' + n; }
 
            // partial rendering function
            options.partial = function(){
              withPartials = true;
              templateCounter++;

              var args = arguments
                , name = args[0]
                , callback = typeof args[1] == 'function' ? args[1] : args[2]
                , locals = typeof args[1] == 'object' ? args[1] : {}

              // mixing options
              for(var k in locals)
                options[k] = locals[k];

              var renderId = getId(renNumber);
              renNumber++;

              var partialFilename = _ops.partials + name + '.' + _ops['view engine'];

              jade.renderFile(partialFilename, { locals: options }, function(err, html) {
                if (err) return saveFile(err);

                rendersStack[renderId] = {html: html}
                --templateCounter || saveFile();
              });

              return '{' + renderId + '}';
            }

            var saveFile = function (err) {
              if (err) return callback(err);

              // replace partials
              if (withPartials)
                for(var id in rendersStack)
                  renderedFile = renderedFile.replace('{' + id + '}', rendersStack[id].html);

              fs.writeFile(saveFilename, renderedFile, _ops.encoding, function(err) {
                if (err) return callback(err);

                self.log('save file:', saveFilename);
                --fileCounter || (function () {
                  self.generated = true;
                  self.compress(callback);
                })();
              });
            }

            var renderFile = function (name, options, file, callback) {
              if (file.text) return callback(null, file.text);
              jade.renderFile(name, options, callback);
            }

            var viewsFilename =_ops.views + template;
            // render html file
            renderFile(viewsFilename, { locals: options }, file, function(err, html) {
              if (err) return callback(err);

              self.log('render file:', viewsFilename);

              renderLayout(file.options.layout, options, html, function(err, htmlLayout) {
                if (err) return callback(err);

                renderedFile = html;

                // replace html by htmlLayout
                if (htmlLayout) {
                  renderedFile = htmlLayout;
                  self.log('decoration: insert html into layout');
                }
              
                // create folders for file ?
                var rePath = file.name.match(/(.+)\/(\w*$)/)
                  , addFolder = rePath ? self.bundlePath + '/' + rePath[1] : false;

                self.createFolder(addFolder, function(err, folder) {
                  if (err) return callback(err);

                  // save html in file (without partials)
                  --templateCounter || saveFile();
                });
              });
            });
          });

        })(self.files[filename]);

      };

    });

  }
};

/**
 * Generates a bundle and saves it to a custom file
 *
 * @api private
 */
 
Bundle.prototype.save = function (params, customFile, callback) {
  var self = this;
  
  this.generate(params, function(err, file){
    if (err) return callback(err);

    var withPath = customFile.match(/[\w|\/].*\//)
      , path = withPath.length ? withPath[0].substr(0, withPath[0].length - 1) : null;

    self.createFolder(path, function(err, folder) {
      if (err) return callback(err);
      var mvFileCmd = 'mv ' + escape(self.filename) + ' ' + customFile;

      exec(mvFileCmd, function (err, stdout, stderr) {
        if (err) return callback(err);

        self.log('moving file:', mvFileCmd);

        callback(null, customFile);
      });

    });

  });
};

/**
 * Generates a bundle and gets the buffer
 *
 * @param private
 */
 
Bundle.prototype.get = function (params, callback) {
  var self = this;
  this.generate(params, function() {

    // reads the file ...
    fs.readFile(self.filename, function(err, data) {
      if (err) return callback(err);

      // sets this.generated = false
      self.generated = false;

      // removes the file
      exec('rm ' + escape(self.bundlePath) + ' -R', function (err, stdout, stderr) {
        if (err) return callback(err);
        
        // ... and passes the buffer to fn
        callback(null, data);
      });
      
    });

  });

};

/**
 * log() util method
 *
 * @api private
 */
 
Bundle.prototype.log = function () {
  if (!this.options.addLogFile) return null;
  
  if (!this.objLog) {
    this.objLog = new log(log.INFO, fs.createWriteStream(this.bundlePath + '/' + this.options.tmpPackname + '.log'));
  }
  
  this.objLog.info(Array.prototype.join.call(arguments, ' '));
}

/**
 * createFolder() util method
 *
 * @api private
 */

Bundle.prototype.createFolder = function(path, fn) {
  if (!path) return fn(null, '');

  exec('mkdir ' + escape(path) + ' -p', function (err, stdout, stderr) {
    if (err) return fn(err);
    fn(null, path);
  });
}

/**
 * Creates a temporary folder.
 *
 * @api private
 */
 
Bundle.prototype.cleanTmpFolder = function (callback) {
  if(!callback) return;

  exec('rm ' + escape(this.bundlePath) + ' -R', function(err, stdout, stderr) {
    if (err) return callback(err);
    if (stderr) return callback(err, stderr);

    if (callback) callback();
  })
}

/**
 * Escape the given shell `arg`.
 *
 * @param {String} arg
 * @return {String}
 * @api private
 */

function escape (arg) {
  return '"' + String(arg).replace(/"/g, '\\"') + '"';
};
